from os import urandom, getenv
from os.path import exists, join, dirname, realpath
from hashlib import sha512

import sqlite3
from flask import Flask, render_template, request, g, session, url_for


def setup_db(filename):
    connection = sqlite3.connect(filename)
    cur = connection.cursor()
    cur.execute('''CREATE TABLE users
                   (
                      id INTEGER PRIMARY KEY,
                      username TEXT NOT NULL,
                      pwhash TEXT NOT NULL
                   )
                ''')
    connection.commit()
    connection.close()


BASEURL = getenv('BASEURL', '')
DBFILE = join(dirname(realpath(__file__)), '.db.db')
SALT = 'justsomerandombytes'.encode()
if not exists(DBFILE):
    setup_db(DBFILE)
app = Flask(__name__)
app.secret_key = urandom(32)


def get_url(endpoint):
    return f'{BASEURL}{url_for(endpoint)}'
app.jinja_env.globals.update(get_url=get_url)


@app.before_request
def init_database():
    g.db = sqlite3.connect(DBFILE)
    g.db.row_factory = sqlite3.Row


@app.teardown_request
def close_database(exception):
    db = getattr(g, 'db', None)
    if db is not None:
        db.close()


@app.route('/', methods=['GET', 'POST'])
def index():
    if request.method == 'POST':
        cur = g.db.cursor()
        cur.execute('SELECT * FROM users WHERE username=? AND pwhash=?', 
                    [request.form['username'], sha512(request.form['password'].encode()+SALT).hexdigest()])
        query = cur.fetchone()
        if not query:
            return render_template('login.html', alert='Invalid credentials!')
        else:
            session['logged_in'] = True
            session['username'] = request.form['username']
            return render_template('internal.html')
    
    if session.get('logged_in'):
        return render_template('internal.html')
    return render_template('login.html')


@app.route('/register', methods=['GET', 'POST'])
def register():
    if request.method == 'POST':
        cur = g.db.cursor()

        if not request.form['username'] or not request.form['password'] or not request.form['passwordconfirm']:
            return render_template('register.html', alert='You need to fill everything.')
        if request.form['password'] != request.form['passwordconfirm']:
            return render_template('register.html', alert='Passwords do not match! Please try again.')

        cur.execute('SELECT * FROM users WHERE username=?', [request.form['username']])
        if cur.fetchone() is not None:
            return render_template('register.html', alert='Username already in use.')

        cur.execute('INSERT INTO users(username, pwhash) VALUES(?,?)', 
                    [request.form['username'], 
                     sha512(request.form['password'].encode()+SALT).hexdigest()])
        g.db.commit()

        return render_template('login.html', success='Account "{}" successfully registered. You can log in now!'.format(request.form['username']))

    return render_template('register.html')


@app.route('/logout')
def logout():
    try:
        session.pop('logged_in')
        session.pop('username')
    except KeyError:
        pass
    return render_template('login.html')


@app.errorhandler(401)
@app.errorhandler(404)
@app.route('/error')
def error(error):
    return render_template('error.html', error=error), error.code


# 500 needs a separate handler, as Flask would print
# the actual piece of code that caused the exception
# for some bizarre reason
@app.errorhandler(500)
@app.route('/error')
def servererror(error):
    return render_template('error.html', error='500: Internal server error'), 500


if __name__ == '__main__':
    app.run(host='127.0.0.1', debug=False, port=6666)
